# coding=utf-8
from earthdiagnostics.diagnostic import Diagnostic, DiagnosticOption, DiagnosticDomainOption, \
    DiagnosticVariableListOption
from earthdiagnostics.modelingrealm import ModelingRealm
from earthdiagnostics.utils import Utils, TempFile
import numpy as np


class SimplifyDimensions(Diagnostic):
    """
    Convert i j files to lon lat when there is no interpolation required,
    i.e. lon is constant over i and lat is constat over j

    :original author: Javier Vegas-Regidor<javier.vegas@bsc.es>

    :created: April 2017

    :param data_manager: data management object
    :type data_manager: DataManager
    :param startdate: startdate
    :type startdate: str
    :param member: member number
    :type member: int
    :param chunk: chunk's number
    :type chunk: int
    :param variable: variable's name
    :type variable: str
    :param domain: variable's domain
    :type domain: ModelingRealm
    """

    alias = 'simdim'
    "Diagnostic alias for the configuration file"

    def __init__(self, data_manager, startdate, member, chunk, domain, variable, grid):
        Diagnostic.__init__(self, data_manager)
        self.startdate = startdate
        self.member = member
        self.chunk = chunk
        self.variable = variable
        self.domain = domain
        self.grid = grid

    def __str__(self):
        return 'Simplify dimension Startdate: {0} Member: {1} Chunk: {2} ' \
               'Variable: {3}:{4} Grid: {5}'.format(self.startdate, self.member, self.chunk, self.domain, self.variable,
                                                    self.grid)

    def __eq__(self, other):
        return self.startdate == other.startdate and self.member == other.member and self.chunk == other.chunk and \
            self.domain == other.domain and self.variable == other.variable and self.grid == self.grid

    @classmethod
    def generate_jobs(cls, diags, options):
        """
            Creates a job for each chunk to compute the diagnostic

            :param diags: Diagnostics manager class
            :type diags: Diags
            :param options: domain,variables,grid
            :type options: list[str]
            :return:
            """
        options_available = (DiagnosticDomainOption(),
                             DiagnosticVariableListOption(diags.data_manager.config.var_manager, 'variables'),
                             DiagnosticOption('grid', ''))
        options = cls.process_options(options, options_available)
        job_list = list()
        variables = options['variables']
        for var in variables:
            for startdate, member, chunk in diags.config.experiment.get_chunk_list():

                job_list.append(SimplifyDimensions(diags.data_manager, startdate, member, chunk,
                                                   options['domain'], var, options['grid']))
        return job_list

    def request_data(self):
        self.variable_file = self.request_chunk(self.domain, self.variable, self.startdate, self.member, self.chunk,
                                                grid=self.grid, to_modify=True)

    def declare_data_generated(self):
        self.simplified = self.declare_chunk(self.domain, self.variable, self.startdate, self.member, self.chunk,
                                             grid=self.grid)

    def compute(self):
        """
        Runs the diagnostic
        """
        handler = Utils.openCdf(self.variable_file.local_file)
        if 'i' not in handler.dimensions:
            raise Exception('Variable {0.domain}:{0.variable} does not have i,j dimensions'.format(self))
        lat = handler.variables['lat']
        lat_values = lat[:, 0:1]
        # noinspection PyTypeChecker
        if np.any((lat[:] - lat_values) != 0):
            raise Exception('Latitude is not constant over i dimension for variable '
                            '{0.domain}:{0.variable}'.format(self))

        lon = handler.variables['lon']
        lon_values = lon[0:1, :]
        # noinspection PyTypeChecker
        if np.any((lon[:] - lon) != 0):
            raise Exception('Longitude is not constant over j dimension for variable '
                            '{0.domain}:{0.variable}'.format(self))

        temp = TempFile.get()
        new_file = Utils.openCdf(temp, 'w')
        for dim in handler.dimensions.keys():
            if dim in ('lat', 'lon', 'i', 'j', 'vertices'):
                continue
            Utils.copy_dimension(handler, new_file, dim, new_names={'i': 'lon', 'j': 'lat'})

        new_file.createDimension('lon', handler.dimensions['i'].size)
        new_file.createDimension('lat', handler.dimensions['j'].size)
        new_file.createDimension('vertices', 2)

        for var in handler.variables.keys():
            if var in ('lat', 'lon', 'i', 'j', 'lat_vertices', 'lon_vertices'):
                continue
            Utils.copy_variable(handler, new_file, var, new_names={'i': 'lon', 'j': 'lat'})
            
        self._create_var('lon', lon_values, handler, new_file)
        self._create_var('lat', lat_values, handler, new_file)
        handler.close()
        new_file.close()

        self.simplified.set_local_file(temp)

    @staticmethod
    def _create_var(var_name, var_values, source, destiny):
        old_var = source.variables[var_name]
        new_var = destiny.createVariable(var_name, old_var.dtype, dimensions=(var_name, ))
        new_var[:] = var_values
        Utils.copy_attributes(new_var, old_var)

        vertices_name = '{0}_vertices'.format(var_name)

        if vertices_name in source.variables:
            var_vertices = source.variables[vertices_name]
            if var_name == 'lon':
                vertices_values = var_vertices[0:1, :, 2:]
            else:
                vertices_values = var_vertices[:, 0:1, 1:3]
            new_lat_vertices = destiny.createVariable(vertices_name, var_vertices.dtype,
                                                      dimensions=(var_name, 'vertices'))
            new_lat_vertices[:] = vertices_values
            Utils.copy_attributes(new_lat_vertices, var_vertices)

