# coding=utf-8
from unittest import TestCase

from earthdiagnostics.config import CMORConfig, ConfigException, THREDDSConfig, ReportConfig, ExperimentConfig
from earthdiagnostics.frequency import Frequencies
from earthdiagnostics.modelingrealm import ModelingRealms


class VariableMock(object):
    def __init__(self):
        self.domain = ModelingRealms.ocean
        self.short_name = 'tos'

    def __eq__(self, other):
        return self.domain == other.domain and self.short_name == other.short_name


class VariableManagerMock(object):
    def get_variable(self, alias, silent=False):
        if alias == 'bad':
            return None
        var = VariableMock()
        var.short_name = alias
        return var


class ParserMock(object):

    def __init__(self):
        self._values = {}

    def add_value(self, section, var, value):
        self._values[self.get_var_string(section, var)] = value

    def get_var_string(self, section, var):
        return '{0}:{1}'.format(section, var)

    def get_value(self, section, var, default):
        try:
            return self._values[self.get_var_string(section, var)]
        except KeyError:
            return default

    def get_bool_option(self, section, var, default):
        return self.get_value(section, var, default)

    def get_path_option(self, section, var, default):
        return self.get_value(section, var, default)

    def get_int_option(self, section, var, default=0):
        return self.get_value(section, var, default)


    def get_int_list_option(self, section, var, default=list(), separator=' '):
        try:
            return [int(val) for val in self._values[self.get_var_string(section, var)].split(separator)]
        except KeyError:
            return default

    def get_list_option(self, section, var, default=list(), separator=' '):
        try:
            return [val for val in self._values[self.get_var_string(section, var)].split(separator)]
        except KeyError:
            return default

    def get_option(self, section, var, default=None):
        return self.get_value(section, var, default)


class TestCMORConfig(TestCase):

    def setUp(self):
        self.mock_parser = ParserMock()
        self.var_manager = VariableManagerMock()

    def test_basic_config(self):
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertEquals(config.ocean, True)
        self.assertEquals(config.atmosphere, True)
        self.assertEquals(config.force, False)
        self.assertEquals(config.force_untar, False)
        self.assertEquals(config.use_grib, True)
        self.assertEquals(config.activity, 'CMIP')
        self.assertEquals(config.associated_experiment, 'to be filled')
        self.assertEquals(config.associated_model, 'to be filled')
        self.assertEquals(config.initialization_description, 'to be filled')
        self.assertEquals(config.initialization_method, '1')
        self.assertEquals(config.initialization_number, 1)
        self.assertEquals(config.source, 'to be filled')
        self.assertEquals(config.version, '')
        self.assertEquals(config.physics_version, '1')
        self.assertEquals(config.physics_description, 'to be filled')
        self.assertEquals(config.filter_files, '')
        self.assertEquals(config.default_atmos_grid, 'gr')
        self.assertEquals(config.default_ocean_grid, 'gn')
        self.assertEquals(config.min_cmorized_vars, 10)

    def test_cmorize(self):
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertTrue(config.cmorize(VariableMock()))
        self.assertTrue(config.cmorize(None))

    def test_cmorize_list(self):
        self.mock_parser.add_value('CMOR', 'VARIABLE_LIST', 'ocean:thetao ocean:tos')

        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertTrue(config.cmorize(VariableMock()))

        thetao_mock = VariableMock()
        thetao_mock.domain = ModelingRealms.ocean
        thetao_mock.short_name = 'thetao'
        self.assertTrue(config.cmorize(thetao_mock))

    def test_bad_list(self):
        self.mock_parser.add_value('CMOR', 'VARIABLE_LIST', '#ocean:tos')
        with self.assertRaises(ConfigException):
            CMORConfig(self.mock_parser, self.var_manager)

        self.mock_parser.add_value('CMOR', 'VARIABLE_LIST', 'atmos:tos')
        with self.assertRaises(ConfigException):
            CMORConfig(self.mock_parser, self.var_manager)

        self.mock_parser.add_value('CMOR', 'VARIABLE_LIST', 'ocean:bad')
        with self.assertRaises(ConfigException):
            CMORConfig(self.mock_parser, self.var_manager)

    def test_not_cmorize(self):
        self.mock_parser.add_value('CMOR', 'VARIABLE_LIST', 'ocean:tos')

        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertTrue(config.cmorize(VariableMock()))

        self.assertFalse(config.cmorize(None))

        tas_mock = VariableMock()
        tas_mock.domain = ModelingRealms.atmos
        tas_mock.short_name = 'tas'
        self.assertFalse(config.cmorize(tas_mock))

        thetao_mock = VariableMock()
        thetao_mock.domain = ModelingRealms.ocean
        thetao_mock.short_name = 'thetao'
        self.assertFalse(config.cmorize(thetao_mock))

    def test_comment(self):
        self.mock_parser.add_value('CMOR', 'VARIABLE_LIST', 'ocean:tos #ocean:thetao ')

        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertTrue(config.cmorize(VariableMock()))

        thetao_mock = VariableMock()
        thetao_mock.domain = ModelingRealms.ocean
        thetao_mock.short_name = 'thetao'
        self.assertFalse(config.cmorize(thetao_mock))

        self.mock_parser.add_value('CMOR', 'VARIABLE_LIST', '#ocean:tos ocean:thetao ')
        with self.assertRaises(ConfigException):
            CMORConfig(self.mock_parser, self.var_manager)

    def test_cmorization_chunk(self):
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertTrue(config.chunk_cmorization_requested(1))

    def test_cmorize_only_some_chunks(self):
        self.mock_parser.add_value('CMOR', 'CHUNKS', '3 5')
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertTrue(config.chunk_cmorization_requested(3))
        self.assertTrue(config.chunk_cmorization_requested(5))
        self.assertFalse(config.chunk_cmorization_requested(1))
        self.assertFalse(config.chunk_cmorization_requested(4))
        self.assertFalse(config.chunk_cmorization_requested(6))

    def test_any_required(self):
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertTrue(config.any_required(['tos']))

        self.mock_parser.add_value('CMOR', 'VARIABLE_LIST', 'ocean:tos ocean:thetao')
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertTrue(config.any_required(['tos', 'thetao', 'tas']))
        self.assertTrue(config.any_required(['tos', 'tas']))
        self.assertTrue(config.any_required(['thetao']))

        self.assertFalse(config.any_required(['tas']))

    def test_hourly_vars(self):
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertEquals(config.get_variables(Frequencies.six_hourly), {})

        self.mock_parser.add_value('CMOR', 'ATMOS_HOURLY_VARS', '128,129:1,130:1-2,131:1:10,132:0:10:5')
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertEquals(config.get_variables(Frequencies.six_hourly), {128: None,
                                                                         129: '1',
                                                                         130: '1,2',
                                                                         131: '1,2,3,4,5,6,7,8,9',
                                                                         132: '0,5'})

        self.assertEquals(config.get_levels(Frequencies.six_hourly, 128), None)
        self.assertEquals(config.get_levels(Frequencies.six_hourly, 129), '1')
        self.assertEquals(config.get_levels(Frequencies.six_hourly, 130), '1,2')
        self.assertEquals(config.get_levels(Frequencies.six_hourly, 131), '1,2,3,4,5,6,7,8,9',)
        self.assertEquals(config.get_levels(Frequencies.six_hourly, 132), '0,5')

    def test_daily_vars(self):
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertEquals(config.get_variables(Frequencies.daily), {})

        self.mock_parser.add_value('CMOR', 'ATMOS_DAILY_VARS', '128,129:1,130:1-2,131:1:10,132:0:10:5')
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertEquals(config.get_variables(Frequencies.daily), {128: None,
                                                                    129: '1',
                                                                    130: '1,2',
                                                                    131: '1,2,3,4,5,6,7,8,9',
                                                                    132: '0,5'})

        self.assertEquals(config.get_levels(Frequencies.daily, 128), None)
        self.assertEquals(config.get_levels(Frequencies.daily, 129), '1')
        self.assertEquals(config.get_levels(Frequencies.daily, 130), '1,2')
        self.assertEquals(config.get_levels(Frequencies.daily, 131), '1,2,3,4,5,6,7,8,9',)
        self.assertEquals(config.get_levels(Frequencies.daily, 132), '0,5')

    def test_monthly_vars(self):
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertEquals(config.get_variables(Frequencies.monthly), {})

        self.mock_parser.add_value('CMOR', 'ATMOS_MONTHLY_VARS', '128,129:1,130:1-2,131:1:10,132:0:10:5')
        config = CMORConfig(self.mock_parser, self.var_manager)
        self.assertEquals(config.get_variables(Frequencies.monthly), {128: None,
                                                                      129: '1',
                                                                      130: '1,2',
                                                                      131: '1,2,3,4,5,6,7,8,9',
                                                                      132: '0,5'})

        self.assertEquals(config.get_levels(Frequencies.monthly, 128), None)
        self.assertEquals(config.get_levels(Frequencies.monthly, 129), '1')
        self.assertEquals(config.get_levels(Frequencies.monthly, 130), '1,2')
        self.assertEquals(config.get_levels(Frequencies.monthly, 131), '1,2,3,4,5,6,7,8,9',)
        self.assertEquals(config.get_levels(Frequencies.monthly, 132), '0,5')

    def test_bad_frequency_vars(self):
        config = CMORConfig(self.mock_parser, self.var_manager)
        with self.assertRaises(ValueError):
            self.assertEquals(config.get_variables(Frequencies.climatology), {})


class TestTHREDDSConfig(TestCase):

    def setUp(self):
        self.mock_parser = ParserMock()

    def test_basic_config(self):
        config = THREDDSConfig(self.mock_parser)
        self.assertEquals(config.server_url, '')

    def test_url(self):
        self.mock_parser.add_value('THREDDS', 'SERVER_URL', 'test_url')
        config = THREDDSConfig(self.mock_parser)
        self.assertEquals(config.server_url, 'test_url')


class TestReportConfig(TestCase):

    def setUp(self):
        self.mock_parser = ParserMock()

    def test_basic_config(self):
        config = ReportConfig(self.mock_parser)
        self.assertEquals(config.path, '')
        self.assertEquals(config.maximum_priority, 10)

    def test_path(self):
        self.mock_parser.add_value('REPORT', 'PATH', 'new_path')
        config = ReportConfig(self.mock_parser)
        self.assertEquals(config.path, 'new_path')

    def test_priority(self):
        config = ReportConfig(self.mock_parser)
        self.assertEquals(config.maximum_priority, 3)


class TestExperimentConfig(TestCase):

    def setUp(self):
        self.mock_parser = ParserMock()

    def test_basic_config(self):
        config = ExperimentConfig(self.mock_parser)

        self.assertEquals(config.startdates, [])
        self.assertEquals(config.members, [])
        self.assertEquals(config.chunk_size, 0)
        self.assertEquals(config.num_chunks, 0)

        self.assertEquals(config.atmos_grid, '')
        self.assertEquals(config.atmos_timestep, 6)
        self.assertEquals(config.ocean_timestep, 6)

    def test_cmor_version_required(self):
        self.mock_parser.add_value('CMOR', 'VERSION', '20001101')
        self.mock_parser.add_value('EXPERIMENT', 'DATA_CONVENTION', 'Primavera')
        config = ExperimentConfig(self.mock_parser)
        self.assertEquals(config.path, 'new_path')

    def test_startdates(self):
        self.mock_parser.add_value('EXPERIMENT', 'STARTDATES', '20001101 20011101')
        config = ExperimentConfig(self.mock_parser)
        self.assertEquals(config.startdates, ['20001101', '20011101'])

        self.mock_parser.add_value('EXPERIMENT', 'STARTDATES', '200(0|1)1101')
        config = ExperimentConfig(self.mock_parser)
        self.assertEquals(config.startdates, ['20001101', '20011101'])

        self.mock_parser.add_value('EXPERIMENT', 'STARTDATES', '200[0-2](02|05|08|11)01')
        config = ExperimentConfig(self.mock_parser)
        print(config.startdates)
        self.assertEquals(config.startdates, [u'20000201', u'20000501', u'20000801', u'20001101', u'20010201',
                                              u'20010501', u'20010801', u'20011101', u'20020201', u'20020501',
                                              u'20020801', u'20021101'])


