# coding=utf-8

from mock import Mock
from unittest import TestCase
from earthdiagnostics.variable import CMORTable, VariableAlias, Variable, VariableJsonException
from earthdiagnostics.modelingrealm import ModelingRealms
from earthdiagnostics.constants import Basins
from earthdiagnostics.frequency import Frequencies


class TestCMORTable(TestCase):

    def setUp(self):
        self.frequency = Mock()

    def test_str(self):
        self.assertEquals(str(CMORTable('name', 'm', 'Month YEAR')), 'name')


class TestVariableAlias(TestCase):

    def test_str(self):
        alias = VariableAlias('alias')
        self.assertEquals(str(alias), 'alias')
        alias.basin = 'basin'
        self.assertEquals(str(alias), 'alias Basin: basin')
        alias.grid = 'grid'
        self.assertEquals(str(alias), 'alias Basin: basin Grid: grid')


class TestVariable(TestCase):

    def test_parse_json(self):
        var = Variable()
        json = {'out_name': 'out_name',
                'standard_name': 'standard_name',
                'long_name': 'long_name',
                'modeling_realm': 'ocean',
                'valid_min': 'valid_min',
                'valid_max': 'valid_max',
                'units': 'units',
                }
        var.parse_json(json, 'out_name')

        self.assertEqual(var.short_name, 'out_name')
        self.assertEqual(var.standard_name, 'standard_name')
        self.assertEqual(var.long_name, 'long_name')

        self.assertEqual(var.valid_min, 'valid_min')
        self.assertEqual(var.valid_max, 'valid_max')
        self.assertEqual(var.units, 'units')
        self.assertEqual(var.priority, 1)

        self.assertEqual(var.domain, ModelingRealms.ocean)

    def test_parse_json_no_out_name(self):
        var = Variable()
        json = {'standard_name': 'standard_name',
                'long_name': 'long_name',
                'modeling_realm': 'ocean',
                'valid_min': 'valid_min',
                'valid_max': 'valid_max',
                'units': 'units',
                }
        with self.assertRaises(VariableJsonException):
            var.parse_json(json, 'out_name')

    def test_parse_json_with_priority(self):
        var = Variable()
        json = {'out_name': 'out_name',
                'standard_name': 'standard_name',
                'long_name': 'long_name',
                'modeling_realm': 'ocean',
                'valid_min': 'valid_min',
                'valid_max': 'valid_max',
                'units': 'units',
                'priority': '2',
                }
        var.parse_json(json, 'out_name')

        self.assertEqual(var.short_name, 'out_name')
        self.assertEqual(var.standard_name, 'standard_name')
        self.assertEqual(var.long_name, 'long_name')

        self.assertEqual(var.valid_min, 'valid_min')
        self.assertEqual(var.valid_max, 'valid_max')
        self.assertEqual(var.units, 'units')
        self.assertEqual(var.priority, 2)

        self.assertEqual(var.domain, ModelingRealms.ocean)

    def test_parse_json_with_primavera_priority(self):
        var = Variable()
        json = {'out_name': 'out_name',
                'standard_name': 'standard_name',
                'long_name': 'long_name',
                'modeling_realm': 'ocean',
                'valid_min': 'valid_min',
                'valid_max': 'valid_max',
                'units': 'units',
                'primavera_priority': '2',
                }
        var.parse_json(json, 'out_name')

        self.assertEqual(var.short_name, 'out_name')
        self.assertEqual(var.standard_name, 'standard_name')
        self.assertEqual(var.long_name, 'long_name')

        self.assertEqual(var.valid_min, 'valid_min')
        self.assertEqual(var.valid_max, 'valid_max')
        self.assertEqual(var.units, 'units')
        self.assertEqual(var.priority, 2)

        self.assertEqual(var.domain, ModelingRealms.ocean)

    def test_get_modelling_realm(self):
        var = Variable()
        domain = var.get_modelling_realm(('ocean',))
        self.assertEqual(ModelingRealms.ocean, domain)

        domain = var.get_modelling_realm(('ocean', 'atmos'))
        self.assertEqual(ModelingRealms.ocean, domain)

        domain = var.get_modelling_realm(('ocean', 'ocnBgchem'))
        self.assertEqual(ModelingRealms.ocnBgchem, domain)

        domain = var.get_modelling_realm(('ocean', 'seaIce'))
        self.assertEqual(ModelingRealms.seaIce, domain)

        domain = var.get_modelling_realm(('atmos', 'atmosChem'))
        self.assertEqual(ModelingRealms.atmosChem, domain)

        domain = var.get_modelling_realm(('land', 'landIce'))
        self.assertEqual(ModelingRealms.landIce, domain)

        domain = var.get_modelling_realm(tuple())
        self.assertIsNone(domain)

    def test_parse_csv(self):
        var = Variable()
        var.parse_csv(['not_used', 'out_name', 'standard_name', 'long_name', 'ocean', 'global', 'units',
                       'valid_min', 'valid_max', 'grid', 'Amon: '])
        self.assertEqual(var.short_name, 'out_name')
        self.assertEqual(var.standard_name, 'standard_name')
        self.assertEqual(var.long_name, 'long_name')

        self.assertEqual(var.valid_min, 'valid_min')
        self.assertEqual(var.valid_max, 'valid_max')
        self.assertEqual(var.units, 'units')
        self.assertEqual(var.grid, 'grid')

        self.assertEqual(var.domain, ModelingRealms.ocean)
        self.assertEqual(var.basin, Basins().Global)

    def test_get_table(self):
        var = Variable()
        var.domain = ModelingRealms.atmos
        table = var.get_table(Frequencies.monthly, 'specs')
        self.assertEqual(table.frequency, Frequencies.monthly)
        self.assertEqual(table.name, 'Amon')
        self.assertEqual(table.date, 'December 2013')

    def test_get_table_added(self):
        var = Variable()
        var.domain = ModelingRealms.atmos
        var.add_table(CMORTable('Amon', Frequencies.monthly, 'December 2013'))
        table = var.get_table(Frequencies.monthly, 'specs')
        self.assertEqual(table.frequency, Frequencies.monthly)
        self.assertEqual(table.name, 'Amon')
        self.assertEqual(table.date, 'December 2013')

    def test_get_table_not_added(self):
        var = Variable()
        var.domain = ModelingRealms.atmos
        var.add_table(CMORTable('Amon', Frequencies.monthly, 'December 2013'))
        table = var.get_table(Frequencies.daily, 'specs')
        self.assertEqual(table.frequency, Frequencies.daily)
        self.assertEqual(table.name, 'day')
        self.assertEqual(table.date, 'December 2013')

    def test_get_table_not_matching(self):
        var = Variable()
        with self.assertRaises(ValueError):
            var.get_table(Frequencies.daily, 'specs')
